function [X,E] = RPLA(D,alpha1,alpha2,beta,tol)

% TVSp solves the following problem:
% min_B 0.5||D-X-E||^2 + alpha_1 ||X||_* + alpha_2 ||X||_TV + beta||E||_1
%   Detailed explanation goes here

if nargin < 5
    tol = 1e-4;
end

% disp('RPLA is running ...');
% parameters
X = zeros(size(D));
E = X;
Z = X;
Y = X;
G = zeros(size(D,1)-1,size(D,2)); 
rho = norm(D(:))/10;
maxIter = 500;

% starting iteration
for iter = 1:maxIter
    % 1. update X using singlar value thresholding
    X = SVT((D-E+rho*(Z-Y/rho))/(1+rho),alpha1/(1+rho));
    % 2. update Z using flsa
    Zold = Z;
    for j = 1:size(Z,2)
        v = X(:,j)+Y(:,j)/(rho+eps);
        g = G(:,j); % subgradient of fused term
        [Z(:,j),G(:,j)] = flsa(v,g,0,alpha2/(rho+eps),length(v),100,1e-10,1,6);
    end
    % 3. update E
    E = D - X;
    E = sign(E) .* max( abs(E)-beta, 0);
    % 4. update Y
    Y = Y + rho*(X-Z);
    % 5. check convergence
    PrimRes = norm(X(:)-Z(:)); % primal residual
    DualRes = rho*norm(Z(:)-Zold(:)); % dual residual
    fprintf('Iter %d: PrimRes = %f, DualRes = %f, rho = %f \n',iter,PrimRes,DualRes,rho);
    th = norm(D(:))*tol;
    if  PrimRes < th && DualRes < th
        break
    end
    % 6. varying penalty parameter rho
    % refer to section 3.4.1 of ADMM paper
    if PrimRes>10*DualRes
        rho = 2*rho;
    elseif DualRes>10*PrimRes
        rho = rho/2;
    else
    end
end

end

function Y = SVT(X,lam)
lam = max(lam,0);
[U,W,V] = svd(X,'econ');
VT = V';
w = diag(W);
ind = find(w>lam);
W = diag(w(ind)-lam);
Y = U(:,ind)*W*VT(ind,:);
end


